@@ -6,8 +6,8 @@
      email: rjones@redhat.com
 
      Christophe Troestler
-     email: Christophe.Troestler@umh.ac.be
-     WWW: http://math.umh.ac.be/an/software/
+     email: Christophe.Troestler@umons.ac.be
+     WWW: http://math.umons.ac.be/anum/software/
 
    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License version 2.1 or
@@ -47,6 +47,88 @@ type t = string list list
 (* Specialize to int for speed *)
 let max i j = if (i:int) < j then j else i
 
+(* Enhance the List module with tail rec functions. *)
+module List = struct
+  include List
+
+  (* Implementation of [map] in JSC Core. *)
+  let map_slow l ~f = List.rev (List.rev_map f l)
+
+  let rec count_map ~f l ctr =
+    match l with
+    | [] -> []
+    | [x1] ->
+       let f1 = f x1 in
+       [f1]
+    | [x1; x2] ->
+       let f1 = f x1 in
+       let f2 = f x2 in
+       [f1; f2]
+    | [x1; x2; x3] ->
+       let f1 = f x1 in
+       let f2 = f x2 in
+       let f3 = f x3 in
+       [f1; f2; f3]
+    | [x1; x2; x3; x4] ->
+       let f1 = f x1 in
+       let f2 = f x2 in
+       let f3 = f x3 in
+       let f4 = f x4 in
+       [f1; f2; f3; f4]
+    | x1 :: x2 :: x3 :: x4 :: x5 :: tl ->
+       let f1 = f x1 in
+       let f2 = f x2 in
+       let f3 = f x3 in
+       let f4 = f x4 in
+       let f5 = f x5 in
+       f1 :: f2 :: f3 :: f4 :: f5 :: (if ctr > 1000
+                                      then map_slow ~f tl
+                                      else count_map ~f tl (ctr + 1))
+
+  let map f l = count_map ~f l 0
+
+  (* Implementation of [append] in JSC core. *)
+  let slow_append l1 l2 = List.rev_append (List.rev l1) l2
+
+  let rec count_append l1 l2 count =
+    match l1 with
+    | []               ->                         l2
+    | [x1]             -> x1                   :: l2
+    | [x1; x2]         -> x1 :: x2             :: l2
+    | [x1; x2; x3]     -> x1 :: x2 :: x3       :: l2
+    | [x1; x2; x3; x4] -> x1 :: x2 :: x3 :: x4 :: l2
+    | x1 :: x2 :: x3 :: x4 :: x5 :: tl ->
+       x1 :: x2 :: x3 :: x4 :: x5 :: (if count > 1000
+                                      then slow_append tl l2
+                                      else count_append tl l2 (count + 1))
+
+  let append l1 l2 = count_append l1 l2 0
+
+  (* Tail recursive [combine]. *)
+  let rec rev_combine acc l1 l2 =
+    match l1, l2 with
+    | ([], []) -> acc
+    | (a1::l1, a2::l2) -> rev_combine ((a1, a2) :: acc) l1 l2
+    | (_, _) -> invalid_arg "List.combine"
+
+  let slow_combine l1 l2 = List.rev (rev_combine [] l1 l2)
+
+  let rec count_combine l1 l2 count =
+    match l1, l2 with
+    | ([], []) -> []
+    | ([x1], [y1]) -> [x1, y1]
+    | ([x1; x2], [y1; y2]) -> [x1, y1; x2, y2]
+    | ([x1; x2; x3], [y1; y2; y3]) -> [x1, y1; x2, y2; x3, y3]
+    | ([x1; x2; x3; x4], [y1; y2; y3; y4]) -> [x1, y1; x2, y2; x3, y3; x4, y4]
+    | (x1 :: x2 :: x3 :: x4 :: tl1), (y1 :: y2 :: y3 :: y4 :: tl2) ->
+       (x1, y1) :: (x2, y2) :: (x3, y3) :: (x4, y4)
+       :: (if count > 1000 then slow_combine tl1 tl2
+           else count_combine tl1 tl2 (count + 1))
+    | (_, _) -> invalid_arg "List.combine"
+
+  let combine l1 l2 = count_combine l1 l2 0
+
+end
 
 class type in_obj_channel =
 object
@@ -107,7 +189,7 @@ let of_in_obj ?(separator=',') ?(excel_tricks=true) in_chan = {
   end_of_file = false;
   current_field = Buffer.create 0xFF;
   record = [];
-  record_n = 0;
+  record_n = 0; (* => first record numbered 1 *)
   separator = separator;
   excel_tricks = excel_tricks;
 }
@@ -373,7 +455,7 @@ let next ic =
   ic.record <- [];
   ic.record_n <- ic.record_n + 1; (* the current line being read *)
   let more_fields = ref true
-  and field_no = ref 0 in
+  and field_no = ref 1 in (* the current field being read *)
   while !more_fields do
     more_fields := add_next_field ic !field_no;
     incr field_no;
@@ -548,7 +630,9 @@ let save ?separator ?excel_tricks fname t =
 let lines = List.length
 
 let columns csv =
-  List.fold_left max 0 (List.map List.length csv)
+  let m = ref 0 in
+  List.iter (fun row -> m := max !m (List.length row)) csv;
+  !m
 
 
 let rec dropwhile f = function
@@ -557,12 +641,12 @@ let rec dropwhile f = function
   | xs -> xs
 
 
+let rec empty_row = function
+  | [] -> true
+  | "" :: xs -> empty_row xs
+  | _ :: _ -> false
+
 let trim ?(top=true) ?(left=true) ?(right=true) ?(bottom=true) csv =
-  let rec empty_row = function
-    | [] -> true
-    | x :: _ when x <> "" -> false
-    | _ :: xs -> empty_row xs
-  in
   let csv = if top then dropwhile empty_row csv else csv in
   let csv =
     if right then
@@ -771,7 +855,7 @@ let save_out_readable chan csv =
             let rp =
               try List.combine r1 r2
               with
-                Invalid_argument "List.combine" ->
+                Invalid_argument _ ->
                   failwith (Printf.sprintf "Csv.save_out_readable: internal \
                               error: length r1 = %d, length r2 = %d"
                               (List.length r1) (List.length r2)) in
