@@ -6,8 +6,8 @@
      email: rjones@redhat.com
 
      Christophe Troestler
-     email: Christophe.Troestler@umh.ac.be
-     WWW: http://math.umh.ac.be/an/software/
+     email: Christophe.Troestler@umons.ac.be
+     WWW: http://math.umons.ac.be/anum/software/
 
    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License version 2.1 or
@@ -66,9 +66,12 @@ end
 (** {2 Input} *)
 
 exception Failure of int * int * string
-  (** [Failure(nrecord, nfield, msg)] is raised to indicate a parsing
-      error for the field number [nfield] on the record number
-      [nrecord], the description [msg] says what is wrong. *)
+(** [Failure(nrecord, nfield, msg)] is raised to indicate a parsing
+    error for the field number [nfield] on the record number
+    [nrecord], the description [msg] says what is wrong.  The first
+    record and the first field of a record are numbered [1] (to
+    correspond to the usual spreadsheet numbering but differing from
+    [List.nth] of the OCaml representation). *)
 
 type in_channel
 (** Stateful handle to input CSV files. *)
